/*
 BP_plot.cc
*/

#include "BP_plot.h"

#include <iostream.h>
#include "wt.h"

#include "v3dSystem.h"

///////////////////////////
//	BPPlot		//
/////////////////////////

#define REMOBJ	// Remove objects from simulation instead of hiding.
#define NOPERFORMER	// Turn off Performer optimizations.
//#define FRAMERATE	// Print the frame rate at certain intervals.

const int BPPlot::Waveform_Tess = 3;	// Tesselation of the waveform objects.
const int BPPlot::Lead_Object_Tess = 4;
					// Tesselation of the lead object.

const int BPPlot::Default_Num_Increments = 80;
                // Default number of plot increments on an BP plot object.
const int BPPlot::Minimum_Num_Increments = 10;
                // Minimum number of plot increments on an BP plot object.

// Initial colors for the various parts of an BP plot object.
const v3dColor BPPlot::BP_plot_background_color = v3dColor (0, 0, 255);
const v3dColor BPPlot::BP_plot_waveform_color = v3dColor (255, 255, 255);
const v3dColor BPPlot::BP_plot_frame_color = v3dColor (255, 255, 255);
const v3dColor BPPlot::BP_plot_lead_object_color = v3dColor (0, 255, 0);

///////////////
// CREATORS //
/////////////

BPPlot::BPPlot (const BPPlot& plot)
{
	initialize (plot.BP_driver_, plot.length_, plot.height_,
		plot.depth_, plot.num_increments_);

	paused_ = plot.paused_;

	current_x_ = plot.current_x_;
	last_x_ = plot.last_x_;
	last_y_ = plot.last_y_;
	current_index_ = plot.current_index_;
	hide_index_ = plot.hide_index_;
	last_drv_index_ = plot.last_drv_index_;

	for (int i = 0; i < num_increments_; ++i)
		plot_values_[i] = plot.plot_values_[i];

	MoveAbsolute (plot.GetAbsolutePosition());
	SetAbsoluteOrientation (plot.GetAbsoluteOrientation());
}

// BPPlot::Copy()
//	Creates a copy of the plot and returns a pointer to the new plot.
inline v3dEntity*
BPPlot::Copy() const
{
	return new BPPlot (*this);
}

///////////////////
// MANIPULATORS //
/////////////////

// BPPlot::initialize
//	Initializer for BPPlot object, called by a BPPlot constructor.
//	Registers this object as a receiver of BP data from an
//	BPDataSender object.
// Parameters:
//	float	length
//			Horizontal extent of the BP plot.
//	float	height
//			Vertical extent of the BP plot.
//	float	depth
//			Thickness of the BP object.
//	int	num_increments
//			Number of increments to show along the length of
//			the plot.
void
BPPlot::initialize (BPDriver	*BP_driver,
	float	length, float height, float depth,
	int	num_increments)
{
	// Assign the BP driver.
	BP_driver_ = BP_driver;

	length_ = length;
	height_ = height;
	depth_ = depth;
	num_increments_ = num_increments;
	waveform_radius_ = height_ / 100;

	// Calculate plot variables based on the client's specifications.
	calculate_plot_variables();

	// Request the range of data from the BP sender object, and calculate
	// the range and scale to be used for the plot.
	data_range_ = BP_driver_->GetDataRange();
	calculate_scale_variables();

	// Initialize variables used in determining plotting position.
	current_x_ = increment_length_;
	last_x_ = last_y_ = 0;
	current_index_ = 0;
	last_drv_index_ = 0;

	// Register this object's event handler with the BP data sender.
	RegisterEventHandler(v3dSystem::GetInstance(),
		v3dSystem::SYSTEM_SIMULATION_LOOP, &BP_update_callback, 0);

	paused_ = false;	// The BP plot doesn't start out paused.

        // Create the background and frame.
	create_background_and_frame ();

	// Create the baseline and other necessary objects.
	create_baseline();
}

// BPPlot::Pause
//	Pause the update of the BP plot.  No new data will be shown until
//	the plot is unpaused.
void
BPPlot::Pause()
{
	paused_ = true;
// ADD: pausing!
}

// BPPlot::Unpause
//	Restart the update of the BP plot.
void
BPPlot::Unpause()
{
	paused_ = false;
// ADD: unpausing!
}

// BPPlot::calculate_plot_variables
//	Calculate plot variables based on the client's specifications.
void
BPPlot::calculate_plot_variables()
{
	waveform_radius_ = height_ / 100;
	hide_interval_ = num_increments_ / 8;
	half_depth_ = depth_ / 2;
	lead_object_radius_ = waveform_radius_ * 3;
	frame_width_ = length_ > height_ ? length_ / 40 : height_ / 40;
	background_depth_ = depth_ / 5;
	vert_buffer_ = frame_width_ * 2;
	horiz_buffer_ = length_ / 100;
	baseline_length_ = length_ - 2 * frame_width_ - 2 * horiz_buffer_; 
	baseline_X_ = frame_width_ + horiz_buffer_;
//	baseline_Y_ = height_ / 2;
	baseline_Y_ = frame_width_ + vert_buffer_;
	increment_length_ = baseline_length_ / num_increments_;
}

// BPPlot::calculate_scale_variables
//	Calculate the vertical range and scale of the plot based on plot
//	variables and the range of data sent by the BP data sender.
void
BPPlot::calculate_scale_variables()
{
	vert_plot_range_ = height_ - 2 * frame_width_ - 2 * vert_buffer_;
	BP_DATA_TYPE range = data_range_.maximum_value -
		(data_range_.minimum_value > 0 ? 0 : data_range_.minimum_value);
	if (range > 0)
		vert_scale_ = vert_plot_range_ / range;
	else
		vert_scale_ = vert_plot_range_;
cerr << "max value: " << data_range_.maximum_value <<
	" min value: " << data_range_.minimum_value <<
	" vert scale: " << vert_scale_ << endl;
}

// BPPlot::create_background_and_frame
//	Creates the bounding box, background and frame objects for the BP plot.
void
BPPlot::create_background_and_frame ()
{
	// Create the bounding box that will serve as the root of the object
	// hierarchy for the component objects of the BP.
	bounding_box_ = new v3dBlockObject (length_, height_, depth_);
	bounding_box_->Hide();

	// Create the BP object background.
	background_ = new v3dBlockObject (length_, height_, background_depth_);
	background_->MoveAbsolute (v3dPos (0, 0,
					(depth_ - background_depth_ / 2)));

	// The background is initially hidden.
	HideBackground();

	// Create the BP object frame.
	frame_bottom_ = new v3dBlockObject (length_, frame_width_, depth_);
	frame_top_ = new v3dBlockObject (length_, frame_width_, depth_);
	frame_left_ = new v3dBlockObject (frame_width_, height_, depth_);
	frame_right_ = new v3dBlockObject (frame_width_, height_, depth_);

	frame_bottom_->MoveAbsolute (v3dPos (0, (height_ - frame_width_) / 2,
		0));
	frame_top_->MoveAbsolute (v3dVector (0, (height_ - frame_width_) / -2,
		0));
	frame_left_->MoveAbsolute (v3dVector ((length_ - frame_width_) / -2,
		0, 0));
	frame_right_->MoveAbsolute (v3dVector ((length_ - frame_width_) / 2,
		0, 0));

	// Add the frame and the background to the bounding box.
	bounding_box_->Add (background_);
	bounding_box_->Add (frame_bottom_);
	bounding_box_->Add (frame_top_);
	bounding_box_->Add (frame_left_);
	bounding_box_->Add (frame_right_);

	// Set the colors of the various elements.
	background_->SetColor(BP_plot_background_color);
	frame_bottom_->SetColor(BP_plot_frame_color);
	frame_top_->SetColor(BP_plot_frame_color);
	frame_left_->SetColor(BP_plot_frame_color);
	frame_right_->SetColor(BP_plot_frame_color);

#ifndef NOPERFORMER
	// Specify Performer rendering.
	WTobject_addperformer (
		(WTobject *)background_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_bottom_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_top_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_left_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_right_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
#endif	// NOPERFORMER

	// Set the BP's implementation object to the background's
	// implementation object.
	imp_ = bounding_box_->GetImplement();

	// The frame is initially shown.
	ShowFrame();
}

void
BPPlot::scale_background_and_frame(const v3dVector &factors,
	const v3dPos& point)
{
#ifndef NOPERFORMER
	// Temporarirly disable Performer rendering.
	WTobject_deleteperformer ((WTobject *)background_->GetHandler());
	WTobject_deleteperformer ((WTobject *)frame_bottom_->GetHandler());
	WTobject_deleteperformer ((WTobject *)frame_top_->GetHandler());
	WTobject_deleteperformer ((WTobject *)frame_left_->GetHandler());
	WTobject_deleteperformer ((WTobject *)frame_right_->GetHandler());
#endif	// NOPERFORMER
	
	bounding_box_->Scale (factors, point);
	background_->Scale (factors, point);
	frame_top_->Scale (factors, point);
	frame_bottom_->Scale (factors, point);
	frame_left_->Scale (factors, point);
	frame_right_->Scale (factors, point);

#ifndef NOPERFORMER
	// Reinstate Performer rendering.
	WTobject_addperformer (
		(WTobject *)background_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_bottom_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_top_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_left_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
	WTobject_addperformer (
		(WTobject *)frame_right_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
#endif	// NOPERFORMER
}

void
BPPlot::delete_background_and_frame()
{
//	delete bounding_box_;
	delete background_;
	delete frame_left_;
	delete frame_right_;
	delete frame_top_;
	delete frame_bottom_;
}

// BPPlot::create_baseline
//	Creates the BP plot's baseline and all objects associated with it.
void
BPPlot::create_baseline()
{
	// Create the baseline origin for the waveform.
//	origin_ = new v3dSphereObject (waveform_radius_, Waveform_Tess,
//		Waveform_Tess);
	origin_ = new v3dBlockObject (1, 1, 1);
	origin_->MoveAbsolute (v3dVector(baseline_X_ - length_ / 2,
			-baseline_Y_ + height_ / 2, 0));
#ifdef REMOBJ
	origin_->Disable();
#else
	origin_->Hide();
#endif

	// Add the baseline origin to the background hierarchy.
	bounding_box_->Add (origin_);

	// Create the lead object.
	float	current_X = baseline_X_ - length_ / 2,
		half_increment = increment_length_ / 2;

	lead_object_ = new v3dSphereObject (lead_object_radius_,
		Lead_Object_Tess, Lead_Object_Tess);
	origin_->Add (lead_object_);
//	lead_object_->MoveAbsolute (v3dVector (current_X + half_increment,
//		-baseline_Y_, half_depth_));
	lead_object_->Move (v3dVector (current_X + half_increment, 0, 0));
	lead_object_->SetColor(BP_plot_lead_object_color);
#ifndef NOPERFORMER
	WTobject_addperformer (
		(WTobject *)lead_object_->GetHandler(),
		FALSE, FALSE, FALSE, FALSE, NULL, NULL);
#endif	// NOPERFORMER

	// Create the individual baseline plot objects.
	baseline_array_ = new v3dLineObject * [num_increments_];
	plot_values_ = new BP_DATA_TYPE [num_increments_];
//	WTp3	align_vec = { 1, 0, 0 };
	v3dVector	inc_vec (increment_length_, 0, 0);

//	v3dPos	from_pos (current_X, -baseline_Y_, half_depth_);
	v3dPos	from_pos (current_X, 0, 0);

	for (int ci = 0; ci < num_increments_; ++ci) {
		plot_values_[ci] = 0;
		from_pos.x (current_X);
		baseline_array_[ci]
			= new v3dLineObject (from_pos, from_pos + inc_vec,
				waveform_radius_, Waveform_Tess);
// CHANGE: this!
		baseline_array_[ci]->SetPoints(*origin_, from_pos, from_pos + inc_vec);
#if 0
		baseline_array_[ci]
			= new v3dLineObject (
				((v3dEntity *)origin_)->GetAbsolutePosition(from_pos),
				((v3dEntity *)origin_)->GetAbsolutePosition(from_pos + inc_vec),
				waveform_radius_, Waveform_Tess);
#endif

		// Add a baseline plot object to the baseline origin.
		origin_->Add (baseline_array_[ci]);

#ifdef REMOBJ
		baseline_array_[ci]->Disable();
#else
		baseline_array_[ci]->Hide();
#endif

		// Color the baseline objects.
		baseline_array_[ci]->SetColor(BP_plot_waveform_color);

		current_X += increment_length_;
	}

	lead_object_->SetColor(BP_plot_lead_object_color);
}

void
BPPlot::delete_baseline()
{
	for (int ci = 0; ci < num_increments_; ci++)
		delete baseline_array_[ci];
	delete origin_;
	delete lead_object_;
}

// BPPlot::BP_update_callback
//	Event handler callback function, notified every cycle to request
//	the latest data from the BP driver.
void
BPPlot::BP_update_callback (const v3dEventMessage *message)
{
#ifdef FRAMERATE
	static int frame_count = 0;

	if (++frame_count == 40) {
		cerr << "Frame rate: " << WTuniverse_framerate() << endl;
		frame_count = 0;
	}
#endif	// FRAMERATE
	((BPPlot *)message->GetEventReceiver())->plot_new_data ();
}

// BPPlot::plot_new_data
//	Copies the latest data from the BP driver, then plots any new
//	data on the BP.
void
BPPlot::plot_new_data ()
{
	int current_drv_index = BP_driver_->GetCurrentDataIndex();

	// If there's no new data to be displayed, then just return.
	if (last_drv_index_ == current_drv_index)
		return;

	int	new_index = current_index_;

	// Copy the data from the BP driver.
	do {
		last_drv_index_ =
			BP_driver_->GetNextDataIndex(last_drv_index_);

		plot_values_[new_index++] =
				BP_driver_->data_list[last_drv_index_];
		if (new_index >= num_increments_)
			new_index = 0;
	} while (last_drv_index_ != current_drv_index);

	// Plot the new data.
	plot_data (new_index);

	current_index_ = new_index;
}

// BPPlot::plot_data
//	Plots the BP data on the waveform up to the new index specified.
// Parameters:
//	int new_index
//		The last index of the plot data to be shown.
void
BPPlot::plot_data (int new_index)
{
	float	this_y;
	v3dPos	new_pos,
		last_pos = v3dVector(last_x_, -(last_y_), 0);

	while (current_index_ != new_index) {
		this_y = plot_values_[current_index_] * vert_scale_;
		new_pos = v3dVector(current_x_, -(this_y), 0);

		hide_index_ = (current_index_ + hide_interval_)
			% num_increments_;
		baseline_array_[hide_index_]->Disable();

// cout << "data:" << this_y << " diff: " << diff_y << " ";

		baseline_array_[current_index_]->SetPoints(*origin_,
							last_pos, new_pos);
		baseline_array_[current_index_]->Enable();

		// Color the plot.
		baseline_array_[current_index_]->SetColor(
						BP_plot_waveform_color);

		last_y_ = this_y;
		last_pos = new_pos;
		++current_index_;
		if (current_index_ >= num_increments_) {
			current_index_ = 0;
			current_x_ = increment_length_;
			last_x_ = 0;
			last_pos.x(0);
		} else {
			last_x_ = current_x_;
			current_x_ += increment_length_;
		}
	}
	lead_object_->Move(new_pos);
}

void
BPPlot::scale_baseline (const v3dVector &factors, const v3dPos& point)
{
	origin_->Scale(factors, point);

	float current_x = increment_length_, last_x = 0;
	float last_y = plot_values_[num_increments_ - 1] * vert_scale_;

	for (int i = 0; i < num_increments_; ++i) {
		float	this_y = plot_values_[i] * vert_scale_;
#if 0
		v3dPos	new_pos = origin_->GetAbsolutePosition()
				+ v3dVector(current_x, -(this_y), 0),
			last_pos = origin_->GetAbsolutePosition()
				+ v3dVector(last_x, -(last_y), 0);
#endif
		v3dPos	new_pos = v3dVector(current_x, -(this_y), 0),
			last_pos = v3dVector(last_x, -(last_y), 0);

		delete baseline_array_[i];
		baseline_array_[i]
			= new v3dLineObject (last_pos, new_pos,
				waveform_radius_, Waveform_Tess);

		origin_->Add (baseline_array_[i]);
// CHANGE: this!
		baseline_array_[i]->SetPoints(*origin_, last_pos, new_pos);

//		baseline_array_[i]->SetPoints(last_pos, new_pos);
		last_x = current_x;
		last_y = this_y;
		current_x += increment_length_;
	}

	// Recalculate plotting position variables.
//	last_x_ *= factors.x();
//	current_x_ *= factors.x();
//	if (current_index_ == 0)
//		last_x_ = 0;
//	else
	last_x_ = current_index_ * increment_length_;
	current_x_ = (current_index_+1) * increment_length_;
	last_y_ *= factors.y();
//	last_y_ = plot_values_ [current_index_] * vert_scale_;

	delete lead_object_;
	lead_object_ = new v3dSphereObject (lead_object_radius_,
		Lead_Object_Tess, Lead_Object_Tess);
	origin_->Add(lead_object_);

	lead_object_->Move(v3dVector(last_x_, -(last_y_), 0));
	lead_object_->SetColor(BP_plot_lead_object_color);
//	lead_object_->Move(origin_->GetAbsolutePosition()
//				+ v3dVector(last_x_, -(last_y_), 0));
}

// BPPlot::fix_hide_interval
//	Re-hides objects along the hide interval to fix any errors introduced
//	when the sweep speed was changed.
void
BPPlot::fix_hide_interval ()
{
	hide_interval_ = num_increments_ / 8;

	int	current_index = current_index_;
	for (int i = 0; i < hide_interval_; ++i, ++current_index) {
		if (current_index >= num_increments_)
			current_index = 0;
#ifdef REMOBJ
		baseline_array_[current_index]->Disable();
#else
		baseline_array_[current_index]->Hide();
#endif
	}
}

// BPPlot::extend_baseline
//	Extends the baseline to the number of increments specified.
//	The old baseline array is copied to a new baseline array, and the
//	new elements in the array are initialized with flat baseline
//	elements.
// Parameters:
//	unsigned int	old_increment_num
//		Old number of increments on the the baseline.
//	unsigned int	new_increment_num
//		New number of increments on the the baseline.
void
BPPlot::extend_baseline (unsigned int old_increment_num,
	unsigned int new_increment_num)
{
	ASSERT (new_increment_num >= old_increment_num);

	v3dLineObject	**new_baseline_array
		= new v3dLineObject * [new_increment_num];
	BP_DATA_TYPE	*new_plot_values
		= new BP_DATA_TYPE [new_increment_num];

	// Copy the old baseline elements into the new array.
	for (int i = 0; i < old_increment_num; ++i) {
		new_baseline_array[i] = baseline_array_[i];
		new_plot_values [i] = plot_values_ [i];
	}
	delete [] baseline_array_;
	delete [] plot_values_;

	// Create new, flat baseline objects.
	v3dVector	inc_vec (increment_length_, 0, 0);
	float	current_X = old_increment_num * increment_length_;

	v3dPos	from_pos (current_X, 0, 0);
//	v3dPos	from_pos (current_X, -baseline_Y_, half_depth_);

	for (int ci = old_increment_num; ci < new_increment_num; ++ci) {
		new_plot_values[ci] = 0;
		from_pos.x (current_X);
#if 0
		new_baseline_array[ci]
			= new v3dLineObject (
				((v3dEntity *)origin_)->GetAbsolutePosition(from_pos),
				((v3dEntity *)origin_)->GetAbsolutePosition(from_pos + inc_vec),
				waveform_radius_, Waveform_Tess);
#endif
		new_baseline_array[ci]
			= new v3dLineObject (from_pos, from_pos + inc_vec,
				waveform_radius_, Waveform_Tess);
// CHANGE: this!
		new_baseline_array[ci]->SetPoints(*origin_, from_pos, from_pos + inc_vec);

		// Add a baseline plot object to the baseline origin.
		origin_->Add (new_baseline_array[ci]);

#if 0
#ifdef REMOBJ
		new_baseline_array[ci]->Disable();
#else
		new_baseline_array[ci]->Hide();
#endif
#endif

		// Color the baseline objects.
		new_baseline_array[ci]->SetColor(BP_plot_waveform_color);

		current_X += increment_length_;
	}

	baseline_array_ = new_baseline_array;
	plot_values_ = new_plot_values;
}

// BPPlot::shrink_baseline
//	Shrinks the baseline to the number of increments specified.
//	The old baseline array is copied to a new baseline array, leaving
//	out extra elements farthest from the leading edge of the graph.
// Parameters:
//	unsigned int	old_increment_num
//		Old number of increments on the the baseline.
//	unsigned int	new_increment_num
//		New number of increments on the the baseline.
void
BPPlot::shrink_baseline (unsigned int old_increment_num,
	unsigned int new_increment_num)
{
	ASSERT (new_increment_num <= old_increment_num);

	v3dLineObject	**new_baseline_array
		= new v3dLineObject * [new_increment_num];
	BP_DATA_TYPE	*new_plot_values
		= new BP_DATA_TYPE [new_increment_num];

	// Copy the old baseline elements into the new array.
	int	start_index, finish_index;
	if (current_index_ >= new_increment_num) {
		start_index = current_index_ - new_increment_num + 1;
		finish_index = current_index_ + 1;
		current_index_ = new_increment_num - 1;
		current_x_ = current_index_ * increment_length_;
		last_x_ = (current_index_-1) * increment_length_;
	} else {
		start_index = 0;
		finish_index = new_increment_num;
	}
	for (int j = 0; j < start_index; ++j)
		delete baseline_array_[j];
	for (int i = start_index; i < finish_index; ++i) {
		new_baseline_array[i-start_index] = baseline_array_[i];
		new_plot_values [i-start_index] = plot_values_ [i];
	}
	for (int k = finish_index; k < old_increment_num; ++k)
		delete baseline_array_[k];

	delete [] baseline_array_;
	delete [] plot_values_;

	baseline_array_ = new_baseline_array;
	plot_values_ = new_plot_values;
}

// BPPlot::MoveFrontLowerLeftCorner
//	Move the entire BP plot.
// Parameters:
//	v3dPos&	pos
//			New location for the front lower left corner.
void
BPPlot::MoveFrontLowerLeftCorner (const v3dPos& pos)
{
	GetObject()->MoveAbsolute (v3dVector (pos.x() + length_ / 2,
		pos.y() - height_ / 2,
		pos.z() + depth_ / 2));
}

void
BPPlot::HideFrame ()
{
	frame_left_->Hide();
	frame_right_->Hide();
	frame_top_->Hide();
	frame_bottom_->Hide();
	frame_hidden_ = true;
}

void
BPPlot::ShowFrame ()
{
	frame_left_->Show();
	frame_right_->Show();
	frame_top_->Show();
	frame_bottom_->Show();
	frame_hidden_ = false;
}

// BPPlot::Scale
//	Stretches the BP plot about a point by the given factors.
void
BPPlot::Scale (const v3dVector &factors, const v3dPos& point)
{
	scale_background_and_frame(factors, point);

	v3dDim dim = bounding_box_->GetSize();
	length_ = dim.x();
	height_ = dim.y();
	depth_ = dim.z();

	calculate_plot_variables();
	calculate_scale_variables();

	scale_baseline(factors, point);

	fix_hide_interval();
}

// BPPlot::SetSize
//	Sets the exact size of the BP.
inline void
BPPlot::SetSize (const v3dDim &dimensions)
{
	Scale (v3dVector (dimensions.x() / length_,
		dimensions.y() / height_,
		dimensions.z() / depth_));
}

// BPPlot::ChangeSweepSpeed
//	Increases or decreases the sweep speed of the BP plot by the
//	specified factor.
//	The number of increments on the baseline is either increased or
//	decreased, and the baseline as a whole is scaled up or down.
// Parameters:
//	float	change_factor
//			Factor to change the speed by.  For example,
//				1 = no change,
//				1.05 = 5% faster,
//				0.95 = 5% slower
void
BPPlot::ChangeSweepSpeed (float change_factor)
{
	int new_num_increments = (int)trunc (num_increments_ * change_factor);
	if (new_num_increments < Minimum_Num_Increments)
		new_num_increments = Minimum_Num_Increments;

	if (change_factor > 1)
		extend_baseline (num_increments_, new_num_increments);
	else
		shrink_baseline (num_increments_, new_num_increments);
	num_increments_ = new_num_increments;

	increment_length_ = baseline_length_ / new_num_increments;
	scale_baseline (v3dVector (1/change_factor, 1, 1),
				origin_->GetAbsolutePosition());

	fix_hide_interval();
}

// BPPlot::~BPPlot
BPPlot::~BPPlot ()
{
	delete_baseline();
	delete_background_and_frame();
}

void
BPPlot::switch_data ()
{
	last_drv_index_ = 0;
	data_range_ = BP_driver_->GetDataRange();
	calculate_scale_variables();
}

